package models

import (
	"time"

	"gitea.pena/PenaSide/discount/internal/proto/discount"
)

type Discount struct {
	Target      DiscountCalculationTarget `json:"target" bson:"target"`
	Condition   DiscountCondition         `json:"condition" bson:"condition"`
	ID          string                    `json:"id" bson:"_id,omitempty"`
	Name        string                    `json:"name" bson:"name"`
	Description string                    `json:"description" bson:"description"`
	Layer       uint32                    `json:"layer" bson:"layer"`
	Audit       Audit                     `json:"audit" bson:"audit"`
	Deprecated  bool                      `json:"deprecated" bson:"deprecated"`
}

func (receiver *Discount) Sanitize() *Discount {
	receiver.ID = ""
	receiver.Deprecated = false
	receiver.Audit = *receiver.Audit.Sanitize()

	return receiver
}

type DiscountCalculationTarget struct {
	Products    []ProductTarget `json:"products" bson:"products"`
	Factor      float64         `json:"factor" bson:"factor"`
	TargetScope TargetScope     `json:"scope" bson:"scope"`
	TargetGroup string          `json:"group" bson:"group"`
	Overhelm    bool            `json:"overhelm" bson:"overhelm"`
}

type DiscountCondition struct {
	Period              *PeriodCondition `json:"period,omitempty" bson:"period,omitempty"`
	Product             *string          `json:"product,omitempty" bson:"product,omitempty"`
	PriceFrom           *uint64          `json:"priceFrom,omitempty" bson:"priceFrom,omitempty"`
	Group               *string          `json:"group,omitempty" bson:"group,omitempty"`
	User                *string          `json:"user,omitempty" bson:"user,omitempty"`
	UserType            *string          `json:"userType,omitempty" bson:"userType,omitempty"`
	Coupon              *string          `json:"coupon,omitempty" bson:"coupon,omitempty"`
	PurchasesAmount     *uint64          `json:"purchasesAmount,omitempty" bson:"purchasesAmount,omitempty"`
	CartPurchasesAmount *uint64          `json:"cartPurchasesAmount,omitempty" bson:"cartPurchasesAmount,omitempty"`

	/* Срок использования (количество дней использования) */
	Term *uint64 `json:"term" bson:"term"`

	/* Количество использований (количество попыток) */
	Usage *uint64 `json:"usage" bson:"usage,omitempty"`
}

type ProductTarget struct {
	ID       string  `json:"productId" bson:"productId"`
	Factor   float64 `json:"factor" bson:"factor"`
	Overhelm bool    `json:"overhelm" bson:"overhelm"`
}

type PeriodCondition struct {
	From time.Time `json:"from" bson:"from"`
	To   time.Time `json:"to" bson:"to"`
}

type TargetScope string

const (
	// TargetSum тип скидки, применяющийся на финальную сформированную сумму всех товаров.
	TargetSum TargetScope = "sum"

	// TargetGroup тип скидки, применяющийся на сумму товаров определённой группы.
	TargetGroup TargetScope = "group"

	// TargetEach тип скидки, применяющийся на определённый товар.
	TargetEach TargetScope = "each"
)

var TargetScopeModelMap = map[discount.TargetScope]TargetScope{
	0: TargetSum,
	1: TargetGroup,
	2: TargetEach,
}

var TargetScopeProtoMap = map[TargetScope]discount.TargetScope{
	"":          0,
	TargetSum:   0,
	TargetGroup: 1,
	TargetEach:  2,
}
