package server

import (
	"context"
	"fmt"
	"net/http"
	"time"

	"github.com/grpc-ecosystem/grpc-gateway/v2/runtime"
	"github.com/sirupsen/logrus"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/proto/discount"
)

type HTTP struct {
	logger *logrus.Logger
	server *http.Server
	mux    *runtime.ServeMux
}

func NewHTTP(logger *logrus.Logger) *HTTP {
	return &HTTP{
		logger: logger,
		mux:    runtime.NewServeMux(),
		server: &http.Server{
			MaxHeaderBytes: 1 << 20,
			ReadTimeout:    20 * time.Second,
			WriteTimeout:   20 * time.Second,
			IdleTimeout:    20 * time.Second,
		},
	}
}

func (receiver *HTTP) Register(ctx context.Context, config *core.GRPCConfiguration) *HTTP {
	options := []grpc.DialOption{grpc.WithTransportCredentials(insecure.NewCredentials())}
	endpoint := fmt.Sprintf("%s:%s", config.Host, config.Port)

	if err := discount.RegisterDiscountServiceHandlerFromEndpoint(ctx, receiver.mux, endpoint, options); err != nil {
		receiver.logger.Errorf("HTTP register error: %v", err)
		panic(err)
	}

	return receiver
}

func (receiver *HTTP) Listen(address string) error {
	receiver.server.Addr = address
	receiver.server.Handler = receiver.mux

	return receiver.server.ListenAndServe()
}

func (receiver *HTTP) Run(config *core.HTTPConfiguration) {
	endpoint := fmt.Sprintf(":%s", config.Port)

	receiver.logger.Infof("HTTP server started on: <%s>", endpoint)

	if err := receiver.Listen(endpoint); err != nil && err != http.ErrServerClosed {
		receiver.logger.Errorf("HTTP listen error: %v", err)
		panic(err)
	}
}
