package utils

import (
	"errors"

	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/models"
	"gitea.pena/PenaSide/discount/internal/proto/discount"
)

func ConstituteConditions(request *discount.ApplyDiscountRequest) (*core.DiscountConditions, error) {
	if request == nil {
		return nil, nil
	}

	// Не пуст ли список привилегий?
	if len(request.GetProducts()) < 1 {
		return nil, nil
	}

	// Создаем массив для условий
	optionalConditions := make([]core.OptionalDiscounCondition, len(request.GetProducts()))
	optionalConditionsIndex := 0

	// Создаем мапу для групповых цен
	var groupPrices = map[string]uint64{}

	// Проходимся по каждой привилегии из запроса
	for _, product := range request.Products {
		if product == nil {
			return nil, errors.New("some of products are nil")
		}

		// Если у привилегии указана группа обновляем групповую цену
		if product.Group != nil {
			groupPrice, ok := groupPrices[*product.Group]

			if !ok {
				groupPrice = 0
			}

			groupPrice += product.Price
			groupPrices[*product.Group] = groupPrice
		}

    if product.Usage != nil {
      if *product.Usage != 0 {
        product.Price = *product.Usage
      }
    }

		optionalConditions[optionalConditionsIndex] = core.OptionalDiscounCondition{
			Product:   &product.ID,
			PriceFrom: &product.Price,
			Group:     product.Group,
			Term:      product.Term,
			Usage:     product.Usage,
		}
		optionalConditionsIndex++
	}

	// Добавляем групповые цены в условия
	for k := range groupPrices {
		var groupName, zero = k, uint64(0)
		optionalConditions = append(optionalConditions, core.OptionalDiscounCondition{
			Group:     &groupName,
			PriceFrom: &zero,
		})
	}

	// Возвращаем объект с условиями скидки
	return &core.DiscountConditions{
		Common: core.CommonDiscountCondition{
			User:            request.UserInformation.ID,
			UserType:        request.UserInformation.Type,
			PurchasesAmount: request.UserInformation.PurchasesAmount,
			Coupon:          request.Coupon,
			Period: models.PeriodCondition{
				From: request.Date.AsTime(),
				To:   request.Date.AsTime(),
			},
		},
		Optionals: optionalConditions,
	}, nil
}
