package discount

import (
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/models"
	"gitea.pena/PenaSide/discount/pkg/convert"
)

/*
TODO:

1) Рефактор всех приватных утилит
2) Покрытие тестами всех утилит
*/

func Calculate(products []core.Product, discounts []models.Discount) uint64 {
	if len(products) < 1 {
		return 0
	}

	price := calculateProducts(products)

	if len(discounts) < 1 {
		return convert.FloatToUint64(price)
	}

	discountTargetsMap := SortOverhelmingDiscounts(discounts)
	productsMap := convert.ArrayToMap(products, func(product core.Product) string {
		return product.ID
	})

	if targetsEach, ok := discountTargetsMap[models.TargetEach]; ok {
		for _, target := range targetsEach {
			productsMap = calculateEachProductWithTarget(productsMap, target)
		}

		price = calculateProductsMap(productsMap)
	}

	if targetsGroup, ok := discountTargetsMap[models.TargetGroup]; ok {
		productGroups := convert.MapToMapArray(productsMap, func(product core.Product) string {
			return product.Group
		})

		pricesGroup := calculateProductsGroup(productGroups)

		for _, target := range targetsGroup {
			pricesGroup[target.TargetGroup] *= target.Factor
		}

		price = calculatePricesGroupSum(pricesGroup)
	}

	if targetsSum, ok := discountTargetsMap[models.TargetSum]; ok {
		for _, target := range targetsSum {
			price *= target.Factor
		}
	}

	return convert.FloatToUint64(price)
}

func calculateProducts(products []core.Product) float64 {
	productsPrice := float64(0)

	for _, product := range products {
		productsPrice += product.Price
	}

	return productsPrice
}

func calculateProductsMap(products map[string]core.Product) float64 {
	productsPrice := float64(0)

	for _, product := range products {
		productsPrice += product.Price
	}

	return productsPrice
}

func calculatePricesGroupSum(pricesGroup map[string]float64) float64 {
	finalPrice := float64(0)

	for _, price := range pricesGroup {
		finalPrice += price
	}

	return finalPrice
}

func calculateEachProductWithTarget(products map[string]core.Product, target models.DiscountCalculationTarget) map[string]core.Product {
	if target.Products == nil && target.Factor == 0 {
		return products
	}

	if target.Products == nil {
		for key, product := range products {
			products[key] = core.Product{
				ID:    key,
				Price: product.Price * target.Factor,
				Group: product.Group,
			}
		}

		return products
	}

	for _, productTarget := range target.Products {
		product, ok := products[productTarget.ID]
		if !ok {
			continue
		}

		if productTarget.Factor != 0 {
			products[product.ID] = core.Product{
				ID:    product.ID,
				Price: product.Price * productTarget.Factor,
				Group: product.Group,
			}

			continue
		}

		products[product.ID] = core.Product{
			ID:    product.ID,
			Price: product.Price * target.Factor,
			Group: product.Group,
		}
	}

	return products
}

func calculateProductsGroup(productsGroup map[string][]core.Product) map[string]float64 {
	pricesGroup := make(map[string]float64)

	for group, products := range productsGroup {
		pricesGroup[group] = calculateProducts(products)
	}

	return pricesGroup
}
