package discount

import (
	"gitea.pena/PenaSide/discount/internal/models"
)

func SortOverhelmingDiscounts(discounts []models.Discount) map[models.TargetScope][]models.DiscountCalculationTarget {
	dividedDiscounts := divideDiscountsByScope(discounts)

	return map[models.TargetScope][]models.DiscountCalculationTarget{
		models.TargetEach:  sortOverhelmingDiscountsEach(dividedDiscounts[models.TargetEach]),
		models.TargetGroup: sortOverhelmingDiscounts(dividedDiscounts[models.TargetGroup]),
		models.TargetSum:   sortOverhelmingDiscounts(dividedDiscounts[models.TargetSum]),
	}
}

func constituteOverhelmingProductTargetLayersMap(discounts []models.Discount) map[string]uint32 {
	overhelmingProductTargetLayers := make(map[string]uint32)

	for _, discount := range discounts {
		for _, productTarget := range discount.Target.Products {
			if existingLayer, ok := overhelmingProductTargetLayers[productTarget.ID]; ok {
				if productTarget.Overhelm && existingLayer < discount.Layer {
					overhelmingProductTargetLayers[productTarget.ID] = discount.Layer
				}

				continue
			}

			if productTarget.Overhelm {
				overhelmingProductTargetLayers[productTarget.ID] = discount.Layer
			}
		}
	}

	return overhelmingProductTargetLayers
}

func constituteOverhelmingDiscountLayer(discounts []models.Discount) uint32 {
	overhelmingDiscountLayer := uint32(0)

	for _, discount := range discounts {
		if discount.Target.Overhelm && overhelmingDiscountLayer < discount.Layer {
			overhelmingDiscountLayer = discount.Layer
		}
	}

	return overhelmingDiscountLayer
}

func sortOverhelmingDiscountsEach(discounts []models.Discount) []models.DiscountCalculationTarget {
	if len(discounts) < 1 {
		return []models.DiscountCalculationTarget{}
	}

	sortedDiscountCalculationTargetsEach := make([]models.DiscountCalculationTarget, 0, len(discounts))

	overhelmingProductTargetLayers := constituteOverhelmingProductTargetLayersMap(discounts)
	overhelmingDiscountLayer := constituteOverhelmingDiscountLayer(discounts)

	for _, discount := range discounts {
		if overhelmingDiscountLayer > discount.Layer {
			continue
		}

		updatedProductTargets := make([]models.ProductTarget, 0, len(discount.Target.Products))

		for _, productTarget := range discount.Target.Products {
			if overhelmingProductTarget, ok := overhelmingProductTargetLayers[productTarget.ID]; ok && overhelmingProductTarget > discount.Layer {
				continue
			}

			updatedProductTargets = append(updatedProductTargets, models.ProductTarget{
				ID:     productTarget.ID,
				Factor: productTarget.Factor,
			})
		}

		sortedDiscountCalculationTargetsEach = append(sortedDiscountCalculationTargetsEach, models.DiscountCalculationTarget{
			Products:    updatedProductTargets,
			Factor:      discount.Target.Factor,
			TargetScope: discount.Target.TargetScope,
			TargetGroup: discount.Target.TargetGroup,
		})
	}

	return sortedDiscountCalculationTargetsEach
}

func sortOverhelmingDiscounts(discounts []models.Discount) []models.DiscountCalculationTarget {
	if len(discounts) < 1 {
		return []models.DiscountCalculationTarget{}
	}

	sortedDiscountCalculationTargets := make([]models.DiscountCalculationTarget, 0, len(discounts))
	overhelmingDiscountLayer := constituteOverhelmingDiscountLayer(discounts)

	for _, discount := range discounts {
		if overhelmingDiscountLayer > discount.Layer {
			continue
		}

		sortedDiscountCalculationTargets = append(sortedDiscountCalculationTargets, models.DiscountCalculationTarget{
			Factor:      discount.Target.Factor,
			TargetScope: discount.Target.TargetScope,
			TargetGroup: discount.Target.TargetGroup,
		})
	}

	return sortedDiscountCalculationTargets
}

func divideDiscountsByScope(discounts []models.Discount) map[models.TargetScope][]models.Discount {
	dividedTargets := make(map[models.TargetScope][]models.Discount, 3)

	for _, discount := range discounts {
		if discount.Target.TargetScope == "" && len(discount.Target.Products) > 0 {
			dividedTargets[models.TargetEach] = append(dividedTargets[models.TargetEach], models.Discount{
				Layer: discount.Layer,
				Target: models.DiscountCalculationTarget{
					Products:    discount.Target.Products,
					Factor:      discount.Target.Factor,
					TargetScope: models.TargetEach,
					Overhelm:    discount.Target.Overhelm,
				},
			})
			continue
		}

		if discount.Target.TargetScope == "" && len(discount.Target.Products) < 1 {
			dividedTargets[models.TargetSum] = append(dividedTargets[models.TargetSum], models.Discount{
				Layer: discount.Layer,
				Target: models.DiscountCalculationTarget{
					Factor:      discount.Target.Factor,
					TargetScope: models.TargetSum,
					Overhelm:    discount.Target.Overhelm,
				},
			})

			continue
		}

		dividedTargets[discount.Target.TargetScope] = append(dividedTargets[discount.Target.TargetScope], discount)
	}

	return dividedTargets
}
