package discount_test

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"gitea.pena/PenaSide/discount/internal/models"
	"gitea.pena/PenaSide/discount/internal/utils/discount"
)

func TestSortOverhelmingDiscounts(t *testing.T) {
	t.Run("Успешная сортировка скидок по каждому продукту при наличии перекрывающией скидки на каждый продукт и на определённый продукт", func(t *testing.T) {
		assert.Equal(t,
			map[models.TargetScope][]models.DiscountCalculationTarget{
				models.TargetEach: {
					{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p1", Factor: 0.93},
							{ID: "p2", Factor: 0.945},
							{ID: "p5", Factor: 0.97},
							{ID: "p7", Factor: 0.83},
						},
					},
					{
						TargetScope: models.TargetEach,
						Factor:      0.8,
						Products: []models.ProductTarget{
							{ID: "p6", Factor: 0.5},
						},
					},
				},
				models.TargetGroup: {},
				models.TargetSum:   {},
			},
			discount.SortOverhelmingDiscounts([]models.Discount{
				{
					ID:    "1",
					Layer: 1,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p7", Factor: 0.94},
						},
					},
				},
				{
					ID:    "2",
					Layer: 2,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p7", Factor: 0.91},
						},
					},
				},
				{
					ID:    "3",
					Layer: 3,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Overhelm:    true,
						Products: []models.ProductTarget{
							{ID: "p1", Factor: 0.93},
							{ID: "p2", Factor: 0.945},
							{ID: "p5", Factor: 0.97},
							{ID: "p6", Factor: 0.97},
							{ID: "p7", Factor: 0.83},
						},
					},
				},
				{
					ID:    "4",
					Layer: 4,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.8,
						Products: []models.ProductTarget{
							{ID: "p6", Factor: 0.5, Overhelm: true},
						},
					},
				},
			}),
		)
	})

	t.Run("Успешная сортировка скидок по каждому продукту при наличии самой перекрывающей скидки", func(t *testing.T) {
		assert.Equal(t,
			map[models.TargetScope][]models.DiscountCalculationTarget{
				models.TargetEach: {
					{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p2", Factor: 0.94},
						},
					},
				},
				models.TargetGroup: {},
				models.TargetSum:   {},
			},
			discount.SortOverhelmingDiscounts([]models.Discount{
				{
					ID:    "5",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Overhelm:    true,
						Products: []models.ProductTarget{
							{ID: "p2", Factor: 0.94},
						},
					},
				},
				{
					ID:    "6",
					Layer: 2,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p7", Factor: 0.91, Overhelm: true},
						},
					},
				},
				{
					ID:    "7",
					Layer: 3,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Overhelm:    true,
						Products: []models.ProductTarget{
							{ID: "p1", Factor: 0.93},
							{ID: "p2", Factor: 0.945},
							{ID: "p5", Factor: 0.97},
							{ID: "p6", Factor: 0.97},
							{ID: "p7", Factor: 0.83},
						},
					},
				},
				{
					ID:    "8",
					Layer: 4,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.8,
						Products: []models.ProductTarget{
							{ID: "p6", Factor: 0.5, Overhelm: true},
						},
					},
				},
			}),
		)
	})

	t.Run("Успешная сортировка скидок по каждому продукту при наличии скидок с одинаковым значением слоя", func(t *testing.T) {
		assert.Equal(t,
			map[models.TargetScope][]models.DiscountCalculationTarget{
				models.TargetEach: {
					{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p2", Factor: 0.94},
						},
					},
					{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p7", Factor: 0.91},
						},
					},
					{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p1", Factor: 0.93},
							{ID: "p2", Factor: 0.945},
							{ID: "p5", Factor: 0.97},
							{ID: "p6", Factor: 0.97},
							{ID: "p7", Factor: 0.83},
						},
					},
				},
				models.TargetGroup: {},
				models.TargetSum:   {},
			},
			discount.SortOverhelmingDiscounts([]models.Discount{
				{
					ID:    "9",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Overhelm:    true,
						Products: []models.ProductTarget{
							{ID: "p2", Factor: 0.94},
						},
					},
				},
				{
					ID:    "1",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p7", Factor: 0.91, Overhelm: true},
						},
					},
				},
				{
					ID:    "2",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Overhelm:    true,
						Products: []models.ProductTarget{
							{ID: "p1", Factor: 0.93},
							{ID: "p2", Factor: 0.945},
							{ID: "p5", Factor: 0.97},
							{ID: "p6", Factor: 0.97},
							{ID: "p7", Factor: 0.83},
						},
					},
				},
				{
					ID:    "3",
					Layer: 4,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.8,
						Products: []models.ProductTarget{
							{ID: "p6", Factor: 0.5, Overhelm: true},
						},
					},
				},
			}),
		)
	})

	t.Run("Успешная сортировка скидок по каждому продукту при наличии скидок с одинаковым значением слоя и полностью перекрывающими другие", func(t *testing.T) {
		assert.Equal(t,
			map[models.TargetScope][]models.DiscountCalculationTarget{
				models.TargetEach: {
					{
						Factor:      0.92,
						TargetScope: models.TargetEach,
						Products: []models.ProductTarget{
							{ID: "p2", Factor: 0.94},
						},
					},
					{
						Factor:      0.92,
						TargetScope: models.TargetEach,
						Products: []models.ProductTarget{
							{ID: "p7", Factor: 0.91},
						},
					},
				},
				models.TargetGroup: {},
				models.TargetSum:   {},
			},
			discount.SortOverhelmingDiscounts([]models.Discount{
				{
					ID:    "4",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Overhelm:    true,
						Products: []models.ProductTarget{
							{ID: "p2", Factor: 0.94},
						},
					},
				},
				{
					ID:    "7",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p7", Factor: 0.91, Overhelm: true},
						},
					},
				},
				{
					ID:    "4",
					Layer: 4,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.8,
						Products: []models.ProductTarget{
							{ID: "p7", Factor: 0.5, Overhelm: true},
						},
					},
				},
			}),
		)
	})

	t.Run("Успешная сортировка скидок по каждому продукту при наличии скидок, где одна полностью перекрывает продукты другой", func(t *testing.T) {
		assert.Equal(t,
			map[models.TargetScope][]models.DiscountCalculationTarget{
				models.TargetEach: {
					{
						TargetScope: models.TargetEach,
						Products: []models.ProductTarget{
							{ID: "p2", Factor: 0.94},
						},
					},
					{
						Factor:      0.92,
						TargetScope: models.TargetEach,
						Products:    []models.ProductTarget{},
					},
				},
				models.TargetGroup: {},
				models.TargetSum:   {},
			},
			discount.SortOverhelmingDiscounts([]models.Discount{
				{
					ID:    "5",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Products: []models.ProductTarget{
							{ID: "p2", Factor: 0.94, Overhelm: true},
						},
					},
				},
				{
					ID:    "6",
					Layer: 4,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p2", Factor: 0.5, Overhelm: true},
						},
					},
				},
			}),
		)
	})

	t.Run("Успешная сортировка скидок по группе продуктов при наличии перекрывающих скидок", func(t *testing.T) {
		assert.Equal(t,
			map[models.TargetScope][]models.DiscountCalculationTarget{
				models.TargetEach: {},
				models.TargetGroup: {
					{
						TargetScope: models.TargetGroup,
						TargetGroup: "123",
						Factor:      0.92,
					},
				},
				models.TargetSum: {},
			},
			discount.SortOverhelmingDiscounts([]models.Discount{
				{
					ID:    "7",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						Overhelm:    true,
						TargetScope: models.TargetGroup,
						TargetGroup: "123",
						Factor:      0.92,
					},
				},
				{
					ID:    "8",
					Layer: 4,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetGroup,
						TargetGroup: "123",
						Factor:      0.96,
					},
				},
				{
					ID:    "9",
					Layer: 3,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetGroup,
						TargetGroup: "123",
						Factor:      0.91,
					},
				},
			}),
		)
	})

	t.Run("Успешная сортировка скидок по группе продуктов при наличии перекрывающих скидок и разных групп", func(t *testing.T) {
		assert.Equal(t,
			map[models.TargetScope][]models.DiscountCalculationTarget{
				models.TargetEach: {},
				models.TargetGroup: {
					{
						TargetScope: models.TargetGroup,
						TargetGroup: "123",
						Factor:      0.92,
					},
				},
				models.TargetSum: {},
			},
			discount.SortOverhelmingDiscounts([]models.Discount{
				{
					ID:    "1",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						Overhelm:    true,
						TargetScope: models.TargetGroup,
						TargetGroup: "123",
						Factor:      0.92,
					},
				},
				{
					ID:    "2",
					Layer: 4,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetGroup,
						TargetGroup: "123",
						Factor:      0.42,
					},
				},
				{
					ID:    "3",
					Layer: 5,
					Target: models.DiscountCalculationTarget{
						Overhelm:    true,
						TargetScope: models.TargetGroup,
						TargetGroup: "111",
						Factor:      0.9,
					},
				},
				{
					ID:    "4",
					Layer: 2,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetGroup,
						TargetGroup: "111",
						Factor:      0.8,
					},
				},
			}),
		)
	})

	t.Run("Успешная сортировка скидок по сумме при наличии скидки, перекрывающая все остальные", func(t *testing.T) {
		assert.Equal(t,
			map[models.TargetScope][]models.DiscountCalculationTarget{
				models.TargetEach:  {},
				models.TargetGroup: {},
				models.TargetSum: {
					{
						TargetScope: models.TargetSum,
						Factor:      0.92,
					},
				},
			},
			discount.SortOverhelmingDiscounts([]models.Discount{
				{
					ID:    "1",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						Overhelm:    true,
						TargetScope: models.TargetSum,
						Factor:      0.92,
					},
				},
				{
					ID:    "2",
					Layer: 4,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetSum,
						Factor:      0.42,
					},
				},
				{
					ID:    "3",
					Layer: 5,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetSum,
						Overhelm:    true,
						Factor:      0.9,
					},
				},
				{
					ID:    "4",
					Layer: 2,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetSum,
						Factor:      0.8,
					},
				},
			}),
		)
	})

	t.Run("Успешная сортировка скидок по сумме при наличии перекрывающей скидки", func(t *testing.T) {
		assert.Equal(t,
			map[models.TargetScope][]models.DiscountCalculationTarget{
				models.TargetEach:  {},
				models.TargetGroup: {},
				models.TargetSum: {
					{
						TargetScope: models.TargetSum,
						Factor:      0.92,
					},
					{
						TargetScope: models.TargetSum,
						Factor:      0.42,
					},
					{
						TargetScope: models.TargetSum,
						Factor:      0.9,
					},
				},
			},
			discount.SortOverhelmingDiscounts([]models.Discount{
				{
					ID:    "1",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetSum,
						Factor:      0.92,
					},
				},
				{
					ID:    "2",
					Layer: 4,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetSum,
						Overhelm:    true,
						Factor:      0.42,
					},
				},
				{
					ID:    "3",
					Layer: 5,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetSum,
						Factor:      0.9,
					},
				},
				{
					ID:    "4",
					Layer: 2,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetSum,
						Factor:      0.8,
					},
				},
			}),
		)
	})

	t.Run("Успешная сортировка смешанных скидок при наличии перекрывающих", func(t *testing.T) {
		assert.Equal(t,
			map[models.TargetScope][]models.DiscountCalculationTarget{
				models.TargetEach: {
					{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p2", Factor: 0.94},
						},
					},
					{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p7", Factor: 0.91},
						},
					},
					{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p1", Factor: 0.93},
							{ID: "p2", Factor: 0.945},
							{ID: "p5", Factor: 0.97},
							{ID: "p6", Factor: 0.97},
							{ID: "p7", Factor: 0.83},
						},
					},
				},
				models.TargetGroup: {
					{
						TargetScope: models.TargetGroup,
						TargetGroup: "123",
						Factor:      0.92,
					},
				},
				models.TargetSum: {
					{
						TargetScope: models.TargetSum,
						Factor:      0.92,
					},
					{
						TargetScope: models.TargetSum,
						Factor:      0.42,
					},
					{
						TargetScope: models.TargetSum,
						Factor:      0.9,
					},
				},
			},
			discount.SortOverhelmingDiscounts([]models.Discount{
				{
					ID:    "1",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetSum,
						Factor:      0.92,
					},
				},
				{
					ID:    "2",
					Layer: 4,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetSum,
						Overhelm:    true,
						Factor:      0.42,
					},
				},
				{
					ID:    "3",
					Layer: 5,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetSum,
						Factor:      0.9,
					},
				},
				{
					ID:    "4",
					Layer: 2,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetSum,
						Factor:      0.8,
					},
				},
				{
					ID:    "5",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						Overhelm:    true,
						TargetScope: models.TargetGroup,
						TargetGroup: "123",
						Factor:      0.92,
					},
				},
				{
					ID:    "6",
					Layer: 4,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetGroup,
						TargetGroup: "123",
						Factor:      0.42,
					},
				},
				{
					ID:    "7",
					Layer: 5,
					Target: models.DiscountCalculationTarget{
						Overhelm:    true,
						TargetScope: models.TargetGroup,
						TargetGroup: "111",
						Factor:      0.9,
					},
				},
				{
					ID:    "8",
					Layer: 2,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetGroup,
						TargetGroup: "111",
						Factor:      0.8,
					},
				},
				{
					ID:    "9",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Overhelm:    true,
						Products: []models.ProductTarget{
							{ID: "p2", Factor: 0.94},
						},
					},
				},
				{
					ID:    "10",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Products: []models.ProductTarget{
							{ID: "p7", Factor: 0.91, Overhelm: true},
						},
					},
				},
				{
					ID:    "11",
					Layer: 6,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.92,
						Overhelm:    true,
						Products: []models.ProductTarget{
							{ID: "p1", Factor: 0.93},
							{ID: "p2", Factor: 0.945},
							{ID: "p5", Factor: 0.97},
							{ID: "p6", Factor: 0.97},
							{ID: "p7", Factor: 0.83},
						},
					},
				},
				{
					ID:    "12",
					Layer: 4,
					Target: models.DiscountCalculationTarget{
						TargetScope: models.TargetEach,
						Factor:      0.8,
						Products: []models.ProductTarget{
							{ID: "p6", Factor: 0.5, Overhelm: true},
						},
					},
				},
			}),
		)
	})
}
