package discount

import (
	"go.mongodb.org/mongo-driver/bson"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/fields"
)

func ConstituteCommonConditions(conditions *core.CommonDiscountCondition, cartPurchasesAmount int64) []bson.M {
	if conditions == nil {
		return []bson.M{}
	}

	constituted := []bson.M{
		{
			fields.DiscountCondition.User: conditions.User,
		},
		{
			fields.DiscountCondition.UserType: conditions.UserType,
		},
	}

	if conditions.PurchasesAmount > 0 {
		constituted = append(constituted, bson.M{
			fields.DiscountCondition.PurchasesAmount: bson.M{
				"$lte": conditions.PurchasesAmount,
				"$ne":  0,
			},
		})
	}

	if cartPurchasesAmount > 0 {
		constituted = append(constituted, bson.M{
			fields.DiscountCondition.CartPurchasesAmount: bson.M{
				"$lte": cartPurchasesAmount,
				"$ne":  0,
			},
		})
	}

	return constituted
}

func ConstituteOptionalConditions(conditions *core.OptionalDiscounCondition) []bson.M {
	if conditions == nil {
		return []bson.M{}
	}

	if conditions.Group != nil && conditions.PriceFrom != nil{
		return []bson.M{
			{
				fields.DiscountCondition.Group:     *conditions.Group,
				fields.DiscountCondition.PriceFrom: bson.M{"$lte": *conditions.PriceFrom},
			},
		}
	}

	return []bson.M{}
}
