package discount

import (
	"go.mongodb.org/mongo-driver/bson"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/fields"
)

func ConditionFilter(conditions *core.DiscountConditions, privilegeAmounts, groupPrices map[string]uint64, layer uint64) *bson.M {
	if conditions == nil {
		return nil
	}

	var cartPurchasesAmount uint64
	for _, val := range groupPrices {
		cartPurchasesAmount = val
	}
	// layer 2,3 тут получается
	expressions := ConstituteCommonConditions(&conditions.Common, int64(cartPurchasesAmount))

	switch layer {
	case 1:
		if conditions.Common.UserType == "nko" {
			return &bson.M{
				fields.DiscountCondition.UserType: "nko",
			}
		} else {
			for product, amount := range privilegeAmounts {
				expressions = append(expressions, bson.M{
					"$and": []bson.M{
						{fields.DiscountCondition.Product: product},
						{fields.DiscountCondition.Term: bson.M{"$lte": amount}},
					{fields.DiscountCondition.User: ""},
					},
				})
			}
			if conditions.Common.User != "" {
				expressions = append(expressions, bson.M{
					"$or": []bson.M{
						{fields.DiscountCondition.User: conditions.Common.User},
					},
				})
			}
		}

	case 2:
		for group, price := range groupPrices {
			expressions = append(expressions, bson.M{
				"$and": []bson.M{
					{fields.DiscountCondition.Group: group},
					{fields.DiscountCondition.PriceFrom: bson.M{"$lte": price}},
					{fields.DiscountCondition.User: ""},
				},
			})
		}
		if conditions.Common.User != "" {
			expressions = append(expressions, bson.M{
				fields.DiscountCondition.User: conditions.Common.User,
			})
		}
	}

	return &bson.M{
		"layer": layer,
		"$or":   expressions,
	}
}
