package discount_test

import (
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/models"
	"gitea.pena/PenaSide/discount/internal/utils/expression/discount"
)

func TestConditionFilter(t *testing.T) {
	productID := "product1"
	productID2 := "product2"
	term := uint64(14)
	usage := uint64(10)
	priceFrom := uint64(13000)
	group := "groupopo"
	userID := "user1"
	userType := "nkvo01001"
	coupon := "coupon"
	purchasesAmount := uint64(10000)
	cartPurchasesAmount := uint64(20000)

	t.Run("Формирование фильтра условий поиска с пустыми условиями", func(t *testing.T) {
		assert.NotNil(t, discount.ConditionFilter(&core.DiscountConditions{}, 1))
	})

	t.Run("Формирование фильтра условий поиска с nil", func(t *testing.T) {
		assert.Nil(t, discount.ConditionFilter(nil, 0))
	})

	t.Run("Формирование фильтра условий поиска со всеми условиями", func(t *testing.T) {
		discount.ConstituteCommonConditions(&core.CommonDiscountCondition{
			Period: models.PeriodCondition{
				From: time.Unix(1674546287, 0).UTC(),
				To:   time.Unix(1674546287, 0).UTC(),
			},
			User:                userID,
			UserType:            userType,
			Coupon:              &coupon,
			PurchasesAmount:     purchasesAmount,
			CartPurchasesAmount: cartPurchasesAmount,
		})

		discount.ConstituteOptionalConditions(&core.OptionalDiscounCondition{
			Product:   &productID,
			Term:      &term,
			Usage:     &usage,
			PriceFrom: &priceFrom,
			Group:     &group,
		})

		discount.ConstituteOptionalConditions(&core.OptionalDiscounCondition{
			Product:   &productID2,
			PriceFrom: &priceFrom,
		})
	})
}
