package discount

import (
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/fields"
	"gitea.pena/PenaSide/discount/pkg/utils"
)

func GetDiscountExpression(settings *core.FilterDiscountSettings) *bson.M {
	if settings == nil || utils.GetFilledFieldsCount(settings) < 2 {
		return nil
	}

	expression := bson.M{}
	commonExpression := getCommonDiscountExpression(settings)
	conditionExpression := getDiscountConditionExpression(settings)
	targetExpression := getDiscountTargetExpression(settings)

	for key, value := range commonExpression {
		expression[key] = value
	}

	for key, value := range conditionExpression {
		expression[key] = value
	}

	for key, value := range targetExpression {
		expression[key] = value
	}

	return &expression
}

func getCommonDiscountExpression(deps *core.FilterDiscountSettings) bson.M {
	expression := bson.M{}

	if deps.ID != nil {
		expression[fields.Discount.ID] = *deps.ID
	}

	if deps.Name != nil {
		expression[fields.Discount.Name] = *deps.Name
	}

	if deps.Description != nil {
		expression[fields.Discount.Description] = *deps.Description
	}

	if deps.Layer != nil {
		expression[fields.Discount.Layer] = *deps.Layer
	}

	if deps.Deprecated != nil {
		expression[fields.Discount.Deprecated] = *deps.Deprecated
	}

	return expression
}

func getDiscountConditionExpression(deps *core.FilterDiscountSettings) bson.M {
	expression := bson.M{}

	if deps.Condition == nil {
		return expression
	}

	if deps.Condition.Period != nil {
		from := primitive.NewDateTimeFromTime(deps.Condition.Period.From)
		to := primitive.NewDateTimeFromTime(deps.Condition.Period.To)

		expression[fields.PeriodCondition.From] = from
		expression[fields.PeriodCondition.To] = to
	}

	if deps.Condition.Product != nil {
		expression[fields.DiscountCondition.Product] = *deps.Condition.Product
	}

	if deps.Condition.PriceFrom != nil {
		expression[fields.DiscountCondition.PriceFrom] = *deps.Condition.PriceFrom
	}

	if deps.Condition.Group != nil {
		expression[fields.DiscountCondition.Group] = *deps.Condition.Group
	}

	if deps.Condition.User != nil {
		expression[fields.DiscountCondition.User] = *deps.Condition.User
	}

	if deps.Condition.UserType != nil {
		expression[fields.DiscountCondition.UserType] = *deps.Condition.UserType
	}

	if deps.Condition.Coupon != nil {
		expression[fields.DiscountCondition.Coupon] = *deps.Condition.Coupon
	}

	if deps.Condition.PurchasesAmount != nil {
		expression[fields.DiscountCondition.PurchasesAmount] = *deps.Condition.PurchasesAmount
	}

	if deps.Condition.CartPurchasesAmount != nil {
		expression[fields.DiscountCondition.CartPurchasesAmount] = *deps.Condition.CartPurchasesAmount
	}

	if deps.Condition.Term != nil {
		expression[fields.DiscountCondition.Term] = *deps.Condition.Term
	}

	if deps.Condition.Usage != nil {
		expression[fields.DiscountCondition.Usage] = *deps.Condition.Usage
	}

	return expression
}

func getDiscountTargetExpression(deps *core.FilterDiscountSettings) bson.M {
	expression := bson.M{}

	if deps.Products != nil {
		expression[fields.DiscountTarget.Products] = *deps.Products
	}

	if deps.Factor != nil {
		expression[fields.DiscountTarget.Factor] = *deps.Factor
	}

	if deps.TargetScope != nil {
		expression[fields.DiscountTarget.TargetScope] = *deps.TargetScope
	}

	if deps.TargetGroup != nil {
		expression[fields.DiscountTarget.TargetGroup] = *deps.TargetGroup
	}

	if deps.Overhelm != nil {
		expression[fields.DiscountTarget.Overhelm] = *deps.Overhelm
	}

	return expression
}
