package discount_test

import (
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/fields"
	"gitea.pena/PenaSide/discount/internal/models"
	"gitea.pena/PenaSide/discount/internal/utils/expression/discount"
)

func TestGetDiscountExpression(t *testing.T) {
	id := "id1"
	name := "discount1oo"
	description := ""
	layer := uint32(3)
	deprecated := false
	user := "testUser3"
	userType := "nkvo090909"
	coupon := "coupon14"
	product := "p12"
	priceFrom := uint64(200)
	cartPurchasesAmount := uint64(20000)
	purchasesAmount := uint64(10000)
	usage := uint64(20)
	term := uint64(40)
	factor := 0.95
	targetScope := models.TargetSum
	targetGroup := "groupffffff"
	overhelm := true
	condition := models.DiscountCondition{
		User:                &user,
		Product:             &product,
		PriceFrom:           &priceFrom,
		Group:               &targetGroup,
		Usage:               &usage,
		UserType:            &userType,
		Coupon:              &coupon,
		CartPurchasesAmount: &cartPurchasesAmount,
		PurchasesAmount:     &purchasesAmount,
		Term:                &term,
		Period: &models.PeriodCondition{
			From: time.Unix(1674546287, 0).UTC(),
			To:   time.Unix(1674546287, 0).UTC(),
		},
	}
	products := []models.ProductTarget{
		{
			ID:       "p1",
			Factor:   0.42,
			Overhelm: false,
		},
	}

	t.Run("Формирование запроса фильтрации с пустыми параметрами", func(t *testing.T) {
		assert.Nil(t, discount.GetDiscountExpression(&core.FilterDiscountSettings{}))
	})

	t.Run("Формирование запроса фильтрации с nil", func(t *testing.T) {
		assert.Nil(t, discount.GetDiscountExpression(nil))
	})

	t.Run("Формирование запроса фильтрации с заполненными параметрами", func(t *testing.T) {
		assert.Equal(t,
			&bson.M{
				fields.Discount.ID:                           id,
				fields.Discount.Name:                         name,
				fields.Discount.Description:                  description,
				fields.Discount.Layer:                        layer,
				fields.Discount.Deprecated:                   deprecated,
				fields.PeriodCondition.From:                  primitive.NewDateTimeFromTime(time.Unix(1674546287, 0).UTC()),
				fields.PeriodCondition.To:                    primitive.NewDateTimeFromTime(time.Unix(1674546287, 0).UTC()),
				fields.DiscountCondition.Product:             product,
				fields.DiscountCondition.PriceFrom:           priceFrom,
				fields.DiscountCondition.PurchasesAmount:     purchasesAmount,
				fields.DiscountCondition.CartPurchasesAmount: cartPurchasesAmount,
				fields.DiscountCondition.Group:               targetGroup,
				fields.DiscountCondition.Usage:               usage,
				fields.DiscountCondition.User:                user,
				fields.DiscountCondition.UserType:            userType,
				fields.DiscountCondition.Coupon:              coupon,
				fields.DiscountCondition.Term:                term,
				fields.DiscountTarget.TargetGroup:            targetGroup,
				fields.DiscountTarget.TargetScope:            targetScope,
				fields.DiscountTarget.Factor:                 factor,
				fields.DiscountTarget.Overhelm:               overhelm,
				fields.DiscountTarget.Products:               products,
			},
			discount.GetDiscountExpression(&core.FilterDiscountSettings{
				ID:          &id,
				Name:        &name,
				Description: &description,
				Layer:       &layer,
				Deprecated:  &deprecated,
				Factor:      &factor,
				TargetScope: &targetScope,
				TargetGroup: &targetGroup,
				Overhelm:    &overhelm,
				Condition:   &condition,
				Products:    &products,
			}),
		)
	})

	t.Run("Формирование запроса фильтрации с отсутствующими параметрами", func(t *testing.T) {
		assert.Equal(t,
			&bson.M{
				fields.Discount.Name:        name,
				fields.Discount.Description: description,
			},
			discount.GetDiscountExpression(&core.FilterDiscountSettings{
				Name:        &name,
				Description: &description,
			}),
		)
	})
}
