package discount

import (
	"time"

	"go.mongodb.org/mongo-driver/bson"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/fields"
	"gitea.pena/PenaSide/discount/pkg/convert"
)

type UpdateDiscountExpression struct {
	ID         string
	Expression *bson.M
}

func GetUpdateDiscountsExpressions(deps []core.UpdateDiscountSettings) []UpdateDiscountExpression {
	discounts := make([]UpdateDiscountExpression, 0, len(deps))
	requestsMap := convert.ArrayToMap(deps, func(element core.UpdateDiscountSettings) string {
		return element.ID
	})

	for _, request := range requestsMap {
		updateExpression := GetUpdateDiscountExpression(&core.UpdateDiscountSettings{
			ID:          request.ID,
			Name:        request.Name,
			Description: request.Description,
			Layer:       request.Layer,
			Deprecated:  request.Deprecated,
			Factor:      request.Factor,
			TargetScope: request.TargetScope,
			TargetGroup: request.TargetGroup,
			Overhelm:    request.Overhelm,
			Condition:   request.Condition,
			Products:    request.Products,
		})

		if updateExpression == nil {
			continue
		}

		discounts = append(discounts, *updateExpression)
	}

	return discounts
}

func GetUpdateDiscountExpression(deps *core.UpdateDiscountSettings) *UpdateDiscountExpression {
	if deps == nil {
		return nil
	}

	updateExpression := GetDiscountExpression(&core.FilterDiscountSettings{
		Condition:   deps.Condition,
		Name:        deps.Name,
		Description: deps.Description,
		Layer:       deps.Layer,
		Deprecated:  deps.Deprecated,
		Products:    deps.Products,
		Factor:      deps.Factor,
		TargetScope: deps.TargetScope,
		TargetGroup: deps.TargetGroup,
		Overhelm:    deps.Overhelm,
	})

	if updateExpression == nil {
		return nil
	}

	updateExpressionCopy := *updateExpression

	if len(updateExpressionCopy) > 0 {
		updateExpressionCopy[fields.Audit.UpdatedAt] = time.Now()
	}

	return &UpdateDiscountExpression{
		ID:         deps.ID,
		Expression: &bson.M{"$set": updateExpressionCopy},
	}
}
