package utils_test

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"gitea.pena/PenaSide/discount/internal/models"
	"gitea.pena/PenaSide/discount/internal/utils"
)

func TestFilterDiscounts(t *testing.T) {
	coupon1 := "testCoupon1"
	coupon2 := "testCoupon2"
	user := "user1"

	t.Run("Корректная фильтрация купоночных скидок", func(t *testing.T) {
		filteredDiscounts, couponDiscount := utils.FilterCouponDiscounts([]models.Discount{
			{Name: "1", Condition: models.DiscountCondition{Coupon: &coupon1}},
			{Name: "11", Condition: models.DiscountCondition{Coupon: &coupon2}},
			{Name: "2", Condition: models.DiscountCondition{User: &user}},
			{Name: "3"},
			{Name: "4"},
		})

		assert.ElementsMatch(t, []models.Discount{
			{Name: "1", Condition: models.DiscountCondition{Coupon: &coupon1}},
			{Name: "2", Condition: models.DiscountCondition{User: &user}},
			{Name: "3"},
			{Name: "4"},
		}, filteredDiscounts)
		assert.Equal(t, &models.Discount{Name: "1", Condition: models.DiscountCondition{Coupon: &coupon1}}, couponDiscount)
	})

	t.Run("Корректная фильтрация купоночных скидок с пустым значением", func(t *testing.T) {
		filteredDiscounts, couponDiscount := utils.FilterCouponDiscounts([]models.Discount{})

		assert.Equal(t, []models.Discount{}, filteredDiscounts)
		assert.Nil(t, couponDiscount)
	})

	t.Run("Корректная фильтрация купоночных скидок без купоночной скидки", func(t *testing.T) {
		filteredDiscounts, couponDiscount := utils.FilterCouponDiscounts([]models.Discount{{}})

		assert.Equal(t, []models.Discount{{}}, filteredDiscounts)
		assert.Nil(t, couponDiscount)
	})
}
