package transfer

import (
	"gitea.pena/PenaSide/discount/internal/models"
	proto "gitea.pena/PenaSide/discount/internal/proto/discount"
)

func DiscountsProtoToModel(discounts *proto.Discounts) []*models.Discount {
	if discounts == nil {
		return []*models.Discount{}
	}

	modelDiscounts := make([]*models.Discount, len(discounts.Discounts))

	for index, discount := range discounts.Discounts {
		modelDiscounts[index] = DiscountProtoToModel(discount)
	}

	return modelDiscounts
}

func DiscountProtoToModel(discount *proto.Discount) *models.Discount {
	if discount == nil {
		return nil
	}

	return &models.Discount{
		ID:          discount.GetID(),
		Name:        discount.GetName(),
		Layer:       discount.GetLayer(),
		Description: discount.GetDescription(),
		Deprecated:  discount.GetDeprecated(),
		Condition:   *DiscountConditionProtoToModel(discount.GetCondition()),
		Target:      *DiscountCalculationTargetProtoToModel(discount.GetTarget()),
		Audit:       *AuditProtoToModel(discount.GetAudit()),
	}
}

func DiscountsModelToProto(discounts []models.Discount) *proto.Discounts {
	protoDiscounts := make([]*proto.Discount, len(discounts))

	for index, discount := range discounts {
		protoDiscounts[index] = DiscountModelToProto(discount)
	}

	return &proto.Discounts{
		Discounts: protoDiscounts,
	}
}

func DiscountModelToProto(discount models.Discount) *proto.Discount {
	return &proto.Discount{
		ID:          discount.ID,
		Name:        discount.Name,
		Layer:       discount.Layer,
		Description: discount.Description,
		Condition:   DiscountConditionModelToProto(&discount.Condition),
		Target:      DiscountCalculationTargetModelToProto(discount.Target),
		Audit:       AuditModelToProto(&discount.Audit),
		Deprecated:  discount.Deprecated,
	}
}
