package transfer_test

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/models"
	"gitea.pena/PenaSide/discount/internal/proto/discount"
	"gitea.pena/PenaSide/discount/internal/utils/transfer"
)

func TestOptionalDiscountProtoToCore(t *testing.T) {
	discountName := "discount-1"
	discountDescription := "discount-description-1"
	discountLayer := uint32(3)
	discountDeprecated := false

	t.Run("перевод скидки с опциональными полями с proto nil в core", func(t *testing.T) {
		assert.Equal(t, &core.DiscountOptional{}, transfer.OptionalDiscountProtoToCore(nil))
	})

	t.Run("перевод скидки с опциональными полями с proto в core", func(t *testing.T) {
		assert.Equal(t,
			&core.DiscountOptional{
				Name:        &discountName,
				Layer:       &discountLayer,
				Description: &discountDescription,
				Condition:   &models.DiscountCondition{},
				Deprecated:  &discountDeprecated,
				Target: &models.DiscountCalculationTarget{
					TargetScope: models.TargetSum,
					Products:    []models.ProductTarget{},
				},
			},
			transfer.OptionalDiscountProtoToCore(&discount.DiscountOptional{
				ID:          "1",
				Name:        &discountName,
				Layer:       &discountLayer,
				Description: &discountDescription,
				Condition:   &discount.DiscountCondition{},
				Target:      &discount.DiscountCalculationTarget{},
				Deprecated:  &discountDeprecated,
			}),
		)
	})

	t.Run("перевод скидки с опциональными полями с proto в core без target", func(t *testing.T) {
		assert.Equal(t,
			&core.DiscountOptional{
				Name:        &discountName,
				Layer:       &discountLayer,
				Description: &discountDescription,
				Condition:   &models.DiscountCondition{},
				Target:      &models.DiscountCalculationTarget{TargetScope: models.TargetSum},
			},
			transfer.OptionalDiscountProtoToCore(&discount.DiscountOptional{
				ID:          "1",
				Name:        &discountName,
				Layer:       &discountLayer,
				Description: &discountDescription,
				Condition:   &discount.DiscountCondition{},
			}),
		)
	})
}
