package transfer_test

import (
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"google.golang.org/protobuf/types/known/timestamppb"
	"gitea.pena/PenaSide/discount/internal/models"
	"gitea.pena/PenaSide/discount/internal/proto/discount"
	"gitea.pena/PenaSide/discount/internal/utils/transfer"
)

/*
TODO:

Переписать тесты с testCases в обычный список t.Run тестов
*/

func TestPeriodProtoToModel(t *testing.T) {
	testCases := []struct {
		name  string
		proto *discount.PeriodCondition
		model *models.PeriodCondition
	}{
		{
			name: "correct transfer period from proto to model",
			proto: &discount.PeriodCondition{
				From: &timestamppb.Timestamp{
					Seconds: 1674546287,
					Nanos:   0,
				},
				To: &timestamppb.Timestamp{
					Seconds: 1674546287,
					Nanos:   0,
				},
			},
			model: &models.PeriodCondition{
				From: time.Unix(1674546287, 0).UTC(),
				To:   time.Unix(1674546287, 0).UTC(),
			},
		},
		{
			name: "correct transfer period with empty milliseconds from proto to model",
			proto: &discount.PeriodCondition{
				From: &timestamppb.Timestamp{
					Seconds: 1674546287,
				},
				To: &timestamppb.Timestamp{},
			},
			model: &models.PeriodCondition{
				From: time.Unix(1674546287, 0).UTC(),
				To:   time.Unix(0, 0).UTC(),
			},
		},
		{
			name: "correct transfer period with empty fields from proto to model",
			proto: &discount.PeriodCondition{
				From: &timestamppb.Timestamp{},
			},
			model: &models.PeriodCondition{
				From: time.Unix(0, 0).UTC(),
				To:   time.Unix(0, 0).UTC(),
			},
		},
	}

	for _, testCase := range testCases {
		t.Run(testCase.name, func(t *testing.T) {
			assert.Equal(t, testCase.model, transfer.PeriodProtoToModel(testCase.proto))
		})
	}
}

func TestPeriodModelToProto(t *testing.T) {
	testCases := []struct {
		name  string
		proto *discount.PeriodCondition
		model *models.PeriodCondition
	}{
		{
			name: "correct transfer period from model to proto",
			proto: &discount.PeriodCondition{
				From: &timestamppb.Timestamp{
					Seconds: 1674546287,
					Nanos:   0,
				},
				To: &timestamppb.Timestamp{
					Seconds: 1674546287,
					Nanos:   0,
				},
			},
			model: &models.PeriodCondition{
				From: time.Unix(1674546287, 0).UTC(),
				To:   time.Unix(1674546287, 0).UTC(),
			},
		},
		{
			name: "correct transfer period with empty milliseconds from model to proto",
			proto: &discount.PeriodCondition{
				From: &timestamppb.Timestamp{
					Seconds: 1674546287,
				},
				To: &timestamppb.Timestamp{},
			},
			model: &models.PeriodCondition{
				From: time.Unix(1674546287, 0).UTC(),
				To:   time.Unix(0, 0).UTC(),
			},
		},
		{
			name: "correct transfer period with empty fields from model to proto",
			proto: &discount.PeriodCondition{
				From: &timestamppb.Timestamp{},
				To:   timestamppb.New(time.Time{}),
			},
			model: &models.PeriodCondition{
				From: time.Unix(0, 0).UTC(),
			},
		},
	}

	for _, testCase := range testCases {
		t.Run(testCase.name, func(t *testing.T) {
			assert.Equal(t, testCase.proto, transfer.PeriodModelToProto(testCase.model))
		})
	}
}
