package transfer

import (
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/models"
	proto "gitea.pena/PenaSide/discount/internal/proto/discount"
)

func ProductTargetsProtoToModel(products []*proto.ProductTarget) []models.ProductTarget {
	modelProducts := make([]models.ProductTarget, len(products))

	for index, product := range products {
		modelProducts[index] = *ProductTargetProtoToModel(product)
	}

	return modelProducts
}

func ProductTargetProtoToModel(product *proto.ProductTarget) *models.ProductTarget {
	if product == nil {
		return nil
	}

	receiveOverhelm := func() bool {
		if product.Overhelm != nil {
			return *product.Overhelm
		}

		return false
	}

	return &models.ProductTarget{
		ID:       product.ID,
		Factor:   product.Factor,
		Overhelm: receiveOverhelm(),
	}
}

func ProductTargetsModelToProto(products []models.ProductTarget) []*proto.ProductTarget {
	protoProducts := make([]*proto.ProductTarget, len(products))

	for index, product := range products {
		protoProducts[index] = ProductTargetModelToProto(product)
	}

	return protoProducts
}

func ProductTargetModelToProto(product models.ProductTarget) *proto.ProductTarget {
	return &proto.ProductTarget{
		ID:       product.ID,
		Factor:   product.Factor,
		Overhelm: &product.Overhelm,
	}
}

func ProductsProtoToCore(productInformations []*proto.ProductInformation) []core.Product {
	products := make([]core.Product, len(productInformations))

	if len(productInformations) < 1 {
		return []core.Product{}
	}

	receiveGroup := func(product *proto.ProductInformation) string {
		if product == nil || product.Group == nil {
			return ""
		}

		return *product.Group
	}

	for index, product := range productInformations {
		if product == nil {
			continue
		}

		products[index] = core.Product{
			ID:    product.ID,
			Price: float64(product.Price),
			Group: receiveGroup(product),
		}
	}

	return products
}
