package transfer_test

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/models"
	"gitea.pena/PenaSide/discount/internal/proto/discount"
	"gitea.pena/PenaSide/discount/internal/utils/transfer"
)

/*
TODO:

1) Переписать тесты с testCases в обычный список t.Run тестов
2) Отформатировать массив объектов
3) Убрать глобальные переменные касающиеся других пакетов
*/

var (
	overhelmTrue = true
)

func TestProductTargetsProtoToModel(t *testing.T) {
	testName := "correct transfer product tagrets from proto to model"
	proto := []*discount.ProductTarget{
		{
			ID:     "1",
			Factor: 20,
		},
		{
			ID:       "2",
			Factor:   20,
			Overhelm: &overhelmTrue,
		},
		{
			ID:     "3",
			Factor: 10,
		},
	}
	model := []models.ProductTarget{
		{
			ID:     "1",
			Factor: 20,
		},
		{
			ID:       "2",
			Factor:   20,
			Overhelm: true,
		},
		{
			ID:     "3",
			Factor: 10,
		},
	}

	t.Run(testName, func(t *testing.T) {
		assert.Equal(t, model, transfer.ProductTargetsProtoToModel(proto))
	})
}

func TestProductTargetProtoToModel(t *testing.T) {
	testCases := []struct {
		name  string
		proto *discount.ProductTarget
		model *models.ProductTarget
	}{
		{
			name: "correct transfer product tagret from proto to model",
			proto: &discount.ProductTarget{
				ID:     "1",
				Factor: 20,
			},
			model: &models.ProductTarget{
				ID:     "1",
				Factor: 20,
			},
		},
		{
			name:  "correct transfer nil product tagret from proto to model",
			proto: nil,
			model: nil,
		},
	}

	for _, testCase := range testCases {
		t.Run(testCase.name, func(t *testing.T) {
			assert.Equal(t, testCase.model, transfer.ProductTargetProtoToModel(testCase.proto))
		})
	}
}

func TestProductTargetsModelToProto(t *testing.T) {
	testName := "correct transfer product tagrets from model to proto"
	proto := []*discount.ProductTarget{
		{
			ID:       "1",
			Factor:   50,
			Overhelm: &overhelmTrue,
		},
		{
			ID:       "2",
			Factor:   20,
			Overhelm: new(bool),
		},
		{
			ID:       "3",
			Factor:   10,
			Overhelm: new(bool),
		},
	}
	model := []models.ProductTarget{
		{
			ID:       "1",
			Factor:   50,
			Overhelm: true,
		},
		{
			ID:     "2",
			Factor: 20,
		},
		{
			ID:     "3",
			Factor: 10,
		},
	}

	t.Run(testName, func(t *testing.T) {
		assert.Equal(t, proto, transfer.ProductTargetsModelToProto(model))
	})
}

func TestProductTargetModelToProto(t *testing.T) {
	testCases := []struct {
		name  string
		proto *discount.ProductTarget
		model models.ProductTarget
	}{
		{
			name: "correct transfer product tagret from model to proto",
			proto: &discount.ProductTarget{
				ID:       "1",
				Factor:   20,
				Overhelm: new(bool),
			},
			model: models.ProductTarget{
				ID:     "1",
				Factor: 20,
			},
		},
		{
			name: "correct transfer nil product tagret from model to proto",
			proto: &discount.ProductTarget{
				Overhelm: new(bool),
			},
			model: models.ProductTarget{},
		},
	}

	for _, testCase := range testCases {
		t.Run(testCase.name, func(t *testing.T) {
			assert.Equal(t, testCase.proto, transfer.ProductTargetModelToProto(testCase.model))
		})
	}
}

func TestProductsProtoToCore(t *testing.T) {
	group := "testGroup"

	testCases := []struct {
		name  string
		proto []*discount.ProductInformation
		core  []core.Product
	}{
		{
			name: "transfer product informations from proto to core",
			proto: []*discount.ProductInformation{
				{
					ID:    "1",
					Price: 20,
					Group: &group,
				},
				{
					ID:    "2",
					Price: 30,
				},
			},
			core: []core.Product{
				{
					ID:    "1",
					Price: 20,
					Group: group,
				},
				{
					ID:    "2",
					Price: 30,
				},
			},
		},
		{
			name:  "transfer empty product informations from proto to core",
			proto: []*discount.ProductInformation{},
			core:  []core.Product{},
		},
		{
			name: "transfer product informations from proto to core with nil proto",
			proto: []*discount.ProductInformation{
				{
					ID:    "1",
					Price: 20,
					Group: &group,
				},
				{
					ID:    "2",
					Price: 30,
				},
				nil,
			},
			core: []core.Product{
				{
					ID:    "1",
					Price: 20,
					Group: group,
				},
				{
					ID:    "2",
					Price: 30,
				},
				{},
			},
		},
	}

	for _, testCase := range testCases {
		t.Run(testCase.name, func(t *testing.T) {
			assert.Equal(t, testCase.core, transfer.ProductsProtoToCore(testCase.proto))
		})
	}
}
