package array_test

import (
	"strings"
	"testing"

	"github.com/stretchr/testify/assert"
	"gitea.pena/PenaSide/discount/pkg/array"
)

func TestFilter(t *testing.T) {
	testCasesWithPrimitives := []struct {
		name          string
		inputArray    []any
		inputCallback func(any, int, []any) bool
		expect        []any
	}{
		{
			name:       "Фильтрация массива строк по значению",
			inputArray: []any{"test10130153", "test2"},
			inputCallback: func(element any, index int, array []any) bool {
				assert.Equal(t, array, []any{"test10130153", "test2"})

				return element == "test10130153"
			},
			expect: []any{"test10130153"},
		},
		{
			name:       "Фильтрация массива строк по индексу",
			inputArray: []any{"test1", "test2"},
			inputCallback: func(element any, index int, array []any) bool {
				assert.Equal(t, array, []any{"test1", "test2"})

				return index == 1
			},
			expect: []any{"test2"},
		},
		{
			name:       "Фильтрация массива чисел по значению",
			inputArray: []any{1, 4},
			inputCallback: func(element any, index int, array []any) bool {
				assert.Equal(t, array, []any{1, 4})

				return element == 1
			},
			expect: []any{1},
		},
		{
			name:       "Фильтрация массива строк по значению с несколькими схожими значениями",
			inputArray: []any{"test1", "test2"},
			inputCallback: func(element any, index int, array []any) bool {
				assert.Equal(t, array, []any{"test1", "test2"})

				return strings.Contains(element.(string), "test")
			},
			expect: []any{"test1", "test2"},
		},
	}

	testCasesWithObjects := []struct {
		name          string
		inputArray    []struct{ Name string }
		inputCallback func(struct{ Name string }, int, []struct{ Name string }) bool
		expect        []struct{ Name string }
	}{
		{
			name: "Фильтрация массива объектов по индексу",
			inputArray: []struct{ Name string }{
				{Name: "test1"},
				{Name: "test2"},
			},
			inputCallback: func(element struct{ Name string }, index int, array []struct{ Name string }) bool {
				assert.Equal(t, array, []struct{ Name string }{
					{Name: "test1"},
					{Name: "test2"},
				})

				return index == 1
			},
			expect: []struct{ Name string }{
				{Name: "test2"},
			},
		},
		{
			name: "Фильтрация массива объектов по значению поля",
			inputArray: []struct{ Name string }{
				{Name: "test5313131"},
				{Name: "test2"},
			},
			inputCallback: func(element struct{ Name string }, index int, array []struct{ Name string }) bool {
				assert.Equal(t, array, []struct{ Name string }{
					{Name: "test5313131"},
					{Name: "test2"},
				})

				return element.Name == "test5313131"
			},
			expect: []struct{ Name string }{
				{Name: "test5313131"},
			},
		},
		{
			name: "Фильтрация массива объектов по совпадению значения поля",
			inputArray: []struct{ Name string }{
				{Name: "test1"},
				{Name: "test2"},
			},
			inputCallback: func(element struct{ Name string }, index int, array []struct{ Name string }) bool {
				assert.Equal(t, array, []struct{ Name string }{
					{Name: "test1"},
					{Name: "test2"},
				})

				return strings.Contains(element.Name, "test")
			},
			expect: []struct{ Name string }{
				{Name: "test1"},
				{Name: "test2"},
			},
		},
	}

	for _, test := range testCasesWithPrimitives {
		t.Run(test.name, func(t *testing.T) {
			assert.Equal(t, test.expect, array.Filter(test.inputArray, test.inputCallback))
		})
	}

	for _, test := range testCasesWithObjects {
		t.Run(test.name, func(t *testing.T) {
			assert.ElementsMatch(t, test.expect, array.Filter(test.inputArray, test.inputCallback))
		})
	}
}
