package convert_test

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"gitea.pena/PenaSide/discount/pkg/convert"
)

func TestArrayToMap(t *testing.T) {
	type Product struct {
		ID    string
		Price float64
		Group string
	}

	t.Run("Успешная конвертация массива в карту по ID", func(t *testing.T) {
		products := []Product{
			{ID: "p1", Price: 1500, Group: "templategen"},
			{ID: "p2", Price: 300, Group: "templategen"},
			{ID: "p5", Price: 30, Group: "dwarfener"},
			{ID: "p6", Price: 70, Group: "dwarfener"},
			{ID: "p7", Price: 800, Group: "dwarfener"},
		}

		assert.Equal(t,
			map[string]Product{
				"p1": {ID: "p1", Price: 1500, Group: "templategen"},
				"p2": {ID: "p2", Price: 300, Group: "templategen"},
				"p5": {ID: "p5", Price: 30, Group: "dwarfener"},
				"p6": {ID: "p6", Price: 70, Group: "dwarfener"},
				"p7": {ID: "p7", Price: 800, Group: "dwarfener"},
			},
			convert.ArrayToMap(products, func(product Product) string {
				return product.ID
			}),
		)
	})
}

func TestMapToMapArray(t *testing.T) {
	type Product struct {
		ID    string
		Price float64
		Group string
	}

	t.Run("Успешная конвертация массива в карту из массивов", func(t *testing.T) {
		productsMap := map[string]Product{
			"p1": {ID: "p1", Price: 1500, Group: "templategen"},
			"p2": {ID: "p2", Price: 300, Group: "templategen"},
			"p5": {ID: "p5", Price: 30, Group: "dwarfener"},
			"p6": {ID: "p6", Price: 70, Group: "dwarfener"},
			"p7": {ID: "p7", Price: 800, Group: "dwarfener"},
		}

		assert.EqualValues(t,
			map[string][]Product{
				"templategen": {
					{ID: "p1", Price: 1500, Group: "templategen"},
					{ID: "p2", Price: 300, Group: "templategen"},
				},
				"dwarfener": {
					{ID: "p5", Price: 30, Group: "dwarfener"},
					{ID: "p6", Price: 70, Group: "dwarfener"},
					{ID: "p7", Price: 800, Group: "dwarfener"},
				},
			},
			convert.MapToMapArray(productsMap, func(product Product) string {
				return product.Group
			}),
		)
	})
}
