package integration_test

import (
	"context"
	"fmt"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"testing"

	"github.com/stretchr/testify/assert"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/proto/discount"
	"gitea.pena/PenaSide/discount/pkg/env"
)

func TestCreateDiscount(t *testing.T) {
	config, err := env.Parse[core.Config]("../../.env.test")

	assert.NoError(t, err)
	assert.NotNil(t, config)

	discountClientConnection, err := grpc.Dial(
		fmt.Sprintf("%s:%s", config.GRPC.Host, config.GRPC.Port),
		grpc.WithTransportCredentials(insecure.NewCredentials()),
	)

	assert.NoError(t, err)

	discountClient := discount.NewDiscountServiceClient(discountClientConnection)

	t.Run("Создание скидки", func(t *testing.T) {
		productID := "p1000"
		term := uint64(40)
		user := primitive.NewObjectID().Hex()

		createdDiscount, err := discountClient.CreateDiscount(
			context.Background(),
			&discount.CreateDiscountRequest{
				Name:        "Новая скидка2",
				Description: "",
				Layer:       3,
				Condition: &discount.DiscountCondition{
					Product: &productID,
					Term:    &term,
					User:    &user,
				},
				Target: &discount.DiscountCalculationTarget{
					Factor: 0.92,
				},
			},
		)

		isNoError := assert.NoError(t, err)
		isNotNil := assert.NotNil(t, createdDiscount)

		if isNoError && isNotNil {
			findedDiscount, err := discountClient.GetDiscountByID(context.Background(), &discount.GetDiscountByIDRequest{
				ID: createdDiscount.ID,
			})

			isNoError := assert.NoError(t, err)
			isNotNil := assert.NotNil(t, findedDiscount)

			if isNoError && isNotNil {
				assert.Equal(t,
					discount.Discount{
						ID:          createdDiscount.ID,
						Name:        createdDiscount.Name,
						Layer:       createdDiscount.Layer,
						Description: createdDiscount.Description,
						Condition:   createdDiscount.Condition,
						Deprecated:  createdDiscount.Deprecated,
					},
					discount.Discount{
						ID:          findedDiscount.ID,
						Name:        findedDiscount.Name,
						Layer:       findedDiscount.Layer,
						Description: findedDiscount.Description,
						Condition:   findedDiscount.Condition,
						Deprecated:  findedDiscount.Deprecated,
					},
				)
			}
		}
	})
}
