package integration_test

import (
	"context"
	"fmt"
	"testing"

	"github.com/stretchr/testify/assert"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/proto/discount"
	"gitea.pena/PenaSide/discount/pkg/env"
)

func TestDeleteDiscount(t *testing.T) {
	config, err := env.Parse[core.Config]("../../.env.test")

	assert.NoError(t, err)
	assert.NotNil(t, config)

	discountClientConnection, err := grpc.Dial(
		fmt.Sprintf("%s:%s", config.GRPC.Host, config.GRPC.Port),
		grpc.WithTransportCredentials(insecure.NewCredentials()),
	)

	assert.NoError(t, err)

	discountClient := discount.NewDiscountServiceClient(discountClientConnection)

	t.Run("Удаление скидки", func(t *testing.T) {
		productID := "p1"
		term := uint64(40)

		createdDiscount, err := discountClient.CreateDiscount(
			context.Background(),
			&discount.CreateDiscountRequest{
				Name:        "Новая скидка, которая скоро удалится",
				Description: "",
				Layer:       3,
				Condition: &discount.DiscountCondition{
					Product: &productID,
					Term:    &term,
				},
				Target: &discount.DiscountCalculationTarget{
					Factor: 0.92,
				},
			},
		)

		isNoError := assert.NoError(t, err)
		isNotNil := assert.NotNil(t, createdDiscount)

		if isNoError && isNotNil {
			deletedDiscount, err := discountClient.DeleteDiscount(context.Background(), &discount.GetDiscountByIDRequest{
				ID: createdDiscount.ID,
			})

			isNoError := assert.NoError(t, err)
			isNotNil := assert.NotNil(t, deletedDiscount)

			if isNoError && isNotNil {
				assert.Equal(t,
					discount.Discount{
						ID:          createdDiscount.ID,
						Name:        createdDiscount.Name,
						Layer:       createdDiscount.Layer,
						Description: createdDiscount.Description,
						Condition:   createdDiscount.Condition,
						Deprecated:  createdDiscount.Deprecated,
					},
					discount.Discount{
						ID:          deletedDiscount.ID,
						Name:        deletedDiscount.Name,
						Layer:       deletedDiscount.Layer,
						Description: deletedDiscount.Description,
						Condition:   deletedDiscount.Condition,
						Deprecated:  deletedDiscount.Deprecated,
					},
				)

				findedDiscount, err := discountClient.GetDiscountByID(context.Background(), &discount.GetDiscountByIDRequest{
					ID: createdDiscount.ID,
				})

				assert.Error(t, err)
				assert.Nil(t, findedDiscount)
			}
		}
	})
}
