package integration_test

import (
	"context"
	"fmt"
	"testing"

	"github.com/stretchr/testify/assert"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/proto/discount"
	"gitea.pena/PenaSide/discount/pkg/env"
	"gitea.pena/PenaSide/discount/tests/integration/data"
	"gitea.pena/PenaSide/discount/tests/integration/utils"
)

/*
TODO:

1) Вынести инициализацию клиента в отдельную утилиту
2) Реализовать ручную миграцию данных для теста, чтобы каждый тест имел свой набор данных
*/

func TestApplyDiscount(t *testing.T) {
	config, err := env.Parse[core.Config]("../../.env.test")

	assert.NoError(t, err)
	assert.NotNil(t, config)

	discountClientConnection, err := grpc.Dial(
		fmt.Sprintf("%s:%s", config.GRPC.Host, config.GRPC.Port),
		grpc.WithTransportCredentials(insecure.NewCredentials()),
	)

	assert.NoError(t, err)

	discountClient := discount.NewDiscountServiceClient(discountClientConnection)

	t.Run("Применение скидок по количеству товаров в корзине", func(t *testing.T) {
		response, err := discountClient.ApplyDiscounts(
			context.Background(),
			&discount.ApplyDiscountRequest{
				UserInformation: &discount.UserInformation{
					ID:                  "buddy",
					Type:                "",
					PurchasesAmount:     3,
					CartPurchasesAmount: 5000,
				},
				Products: []*discount.ProductInformation{
					{
						ID:    "p1",
						Price: 2000,
					},
					{
						ID:    "p2",
						Price: 3000,
					},
				},
			},
		)

		isNoError := assert.NoError(t, err)
		isNotNil := assert.NotNil(t, response)

		if isNoError && isNotNil {
			utils.CleanDiscountsIDs(response.AppliedDiscounts)

			assert.Equal(t, []*discount.Discount{data.DiscountID4}, response.AppliedDiscounts)
			assert.Equal(t, uint64(4925), response.Price)
		}
	})

	t.Run("Применение скидок по количеству товаров в корзине + на один товар", func(t *testing.T) {
		term := uint64(30)

		response, err := discountClient.ApplyDiscounts(
			context.Background(),
			&discount.ApplyDiscountRequest{
				UserInformation: &discount.UserInformation{
					ID:                  "buddy",
					Type:                "",
					PurchasesAmount:     4928.0,
					CartPurchasesAmount: 5105,
				},
				Products: []*discount.ProductInformation{
					{
						ID:    "p1",
						Price: 2000,
					},
					{
						ID:    "p2",
						Price: 3000,
					},
					{
						ID:    "p3",
						Price: 105,
						Term:  &term,
					},
				},
			},
		)

		isNoError := assert.NoError(t, err)
		isNotNil := assert.NotNil(t, response)

		if isNoError && isNotNil {
			utils.CleanDiscountsIDs(response.AppliedDiscounts)

			assert.Equal(t, []*discount.Discount{data.DiscountID4, data.DiscountID22}, response.AppliedDiscounts)
			assert.Equal(t, uint64(5025), response.Price)
		}
	})
}
