package data

import (
	"google.golang.org/protobuf/types/known/timestamppb"
	"gitea.pena/PenaSide/discount/internal/proto/discount"
)

var (
	varFalse = false
	varTrue  = true

	cartPurchasesAmounts = []uint64{5000, 50000}
	purchasesAmounts     = []uint64{10000, 25000, 50000}
	priceFroms           = []uint64{1000, 5000, 2000, 6000, 500, 2500}

	terms  = []uint64{30, 90, 180}
	usages = []uint64{100, 350, 500}

	groups    = []string{"", "templategen", "squiz", "dwarfener"}
	users     = []string{"buddy", "another_buddy"}
	coupons   = []string{"coupon1"}
	products  = []string{"p1", "p2", "p3", "p4", "p5", "p6"}
	userTypes = []string{"nko"}

	periods = []discount.PeriodCondition{
		{
			From: &timestamppb.Timestamp{
				Seconds: 1672444800,
				Nanos:   0,
			},
			To: &timestamppb.Timestamp{
				Seconds: 1704067200,
				Nanos:   0,
			},
		},
	}
)

var (
	DiscountID1 = &discount.Discount{
		Name:        "Лояльность 1",
		Description: "постоянная скидка для юзеров, внёсших на проект от 10 000 рублей. Применяется на итоговую сумму, после скидок за сумму в корзине",
		Layer:       4,
		Condition: &discount.DiscountCondition{
			PurchasesAmount: &purchasesAmounts[0],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Sum.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Factor:      0.99,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID2 = &discount.Discount{
		Name:        "Лояльность 2",
		Description: "постоянная скидка для юзеров, внёсших на проект от 25 000 рублей. Применяется на итоговую сумму, после скидок за сумму в корзине",
		Layer:       4,
		Condition: &discount.DiscountCondition{
			PurchasesAmount: &purchasesAmounts[1],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Sum.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Factor:      0.99,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID3 = &discount.Discount{
		Name:        "Лояльность 3",
		Description: "постоянная скидка для юзеров, внёсших на проект от 50 000 рублей. Применяется на итоговую сумму, после скидок за сумму в корзине",
		Layer:       4,
		Condition: &discount.DiscountCondition{
			PurchasesAmount: &purchasesAmounts[2],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Sum.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Factor:      0.975,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID4 = &discount.Discount{
		Name:        "Корзина 1",
		Description: "Скидка на размер корзины от 5 000 р. Применяется на итоговую сумму, после суммирования корзины",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			CartPurchasesAmount: &cartPurchasesAmounts[0],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Sum.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Factor:      0.985,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID5 = &discount.Discount{
		Name:        "Корзина 2",
		Description: "Скидка на размер корзины от 50 000 р. Применяется на итоговую сумму, после суммирования корзины",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			CartPurchasesAmount: &cartPurchasesAmounts[1],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Sum.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Factor:      0.965,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID6 = &discount.Discount{
		Name:        "НКО",
		Description: "Скидка всем подтвердившим статус НКО. Перекрывает ВСЕ остальные скидки. Если эта скидка срабатывает, остальные можно не вычислять. Т.е. если на уровне 0 находится какая-лидо скидка для выданных условий, просто суммируем корзину и применяем к сумме указанный множитель, после чего прекращаем процесс рассчёта",
		Layer:       1,
		Condition: &discount.DiscountCondition{
			UserType: &userTypes[0],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Sum.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varTrue,
			Factor:      0.2,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID7 = &discount.Discount{
		Name:        "Промокод На АБ тесты",
		Description: "Скидка, полученная конкретным юзером, после введения промокода. Заменяет собой не промокодовую",
		Layer:       2,
		Condition: &discount.DiscountCondition{
			User:   &users[0],
			Coupon: &coupons[0],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Each.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varTrue,
			Products: []*discount.ProductTarget{
				{
					ID:       products[5],
					Factor:   0.5,
					Overhelm: &varFalse,
				},
			},
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID8 = &discount.Discount{
		Name:        "Новогодняя скидка",
		Description: "",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			Period: &periods[0],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Each.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Products: []*discount.ProductTarget{
				{
					ID:       products[0],
					Factor:   0.965,
					Overhelm: &varFalse,
				},
				{
					ID:       products[1],
					Factor:   0.5,
					Overhelm: &varFalse,
				},
				{
					ID:       products[2],
					Factor:   0.8,
					Overhelm: &varFalse,
				},
				{
					ID:       products[3],
					Factor:   0.95,
					Overhelm: &varFalse,
				},
			},
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID9 = &discount.Discount{
		Name:        "Скидка на определённого пользователя",
		Description: "",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			User: &users[1],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Sum.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Factor:      0.95,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID10 = &discount.Discount{
		Name:        "Анлим Шабло 1",
		Description: "Скидка на количество безлимитных дней работы от 30 дней",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			Product: &products[0],
			Term:    &terms[0],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Each.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Products: []*discount.ProductTarget{
				{
					ID:       products[0],
					Factor:   0.975,
					Overhelm: &varFalse,
				},
			},
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID11 = &discount.Discount{
		Name:        "Анлим Шабло 2",
		Description: "Скидка на количество безлимитных дней работы от 90 дней",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			Product: &products[0],
			Term:    &terms[1],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Each.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Products: []*discount.ProductTarget{
				{
					ID:       products[0],
					Factor:   0.975,
					Overhelm: &varFalse,
				},
			},
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID12 = &discount.Discount{
		Name:        "Анлим Шабло 3",
		Description: "Скидка на количество безлимитных дней работы от 180 дней",
		Layer:       1,
		Condition: &discount.DiscountCondition{
			Product: &products[0],
			Term:    &terms[2],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Each.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Products: []*discount.ProductTarget{
				{
					ID:       products[0],
					Factor:   0.93,
					Overhelm: &varFalse,
				},
			},
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID13 = &discount.Discount{
		Name:        "Генерации Шабло 1",
		Description: "Скидка на количество генераций от 100 шт",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			Product: &products[1],
			Usage:   &usages[0],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Each.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Products: []*discount.ProductTarget{
				{
					ID:       products[1],
					Factor:   0.995,
					Overhelm: &varFalse,
				},
			},
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID14 = &discount.Discount{
		Name:        "Генерации Шабло 2",
		Description: "Скидка на количество генераций от 350 шт",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			Product: &products[1],
			Usage:   &usages[1],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Each.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Products: []*discount.ProductTarget{
				{
					ID:       products[1],
					Factor:   0.98,
					Overhelm: &varFalse,
				},
			},
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID15 = &discount.Discount{
		Name:        "Генерации Шабло 3",
		Description: "Скидка на количество генераций от 500 шт",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			Product: &products[1],
			Usage:   &usages[2],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Each.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Products: []*discount.ProductTarget{
				{
					ID:       products[1],
					Factor:   0.945,
					Overhelm: &varFalse,
				},
			},
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID16 = &discount.Discount{
		Name:        "Шаблонизатор 1",
		Description: "Скидка на сумму стоимостей товаров, принадлежащих сервису шаблонизации, от 1000 р",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			PriceFrom: &priceFroms[0],
			Group:     &groups[1],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Group.Enum(),
			TargetGroup: &groups[1],
			Overhelm:    &varFalse,
			Factor:      0.996,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID17 = &discount.Discount{
		Name:        "Шаблонизатор 2",
		Description: "Скидка на сумму стоимостей товаров, принадлежащих сервису шаблонизации, от 5000 р",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			PriceFrom: &priceFroms[1],
			Group:     &groups[1],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Group.Enum(),
			TargetGroup: &groups[1],
			Overhelm:    &varFalse,
			Factor:      0.983,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID18 = &discount.Discount{
		Name:        "Опросник 1",
		Description: "Скидка на сумму стоимостей товаров, принадлежащих сервису опросника, от 2000 р",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			PriceFrom: &priceFroms[2],
			Group:     &groups[2],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Group.Enum(),
			TargetGroup: &groups[2],
			Overhelm:    &varFalse,
			Factor:      0.983,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID19 = &discount.Discount{
		Name:        "Опросник 2",
		Description: "Скидка на сумму стоимостей товаров, принадлежащих сервису опросника, от 6000 р",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			PriceFrom: &priceFroms[3],
			Group:     &groups[2],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Group.Enum(),
			TargetGroup: &groups[2],
			Overhelm:    &varFalse,
			Factor:      0.969,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID20 = &discount.Discount{
		Name:        "Сокращатель 1",
		Description: "Скидка на сумму стоимостей товаров, принадлежащих сервису сокращателя, от 500 р",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			PriceFrom: &priceFroms[4],
			Group:     &groups[3],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Group.Enum(),
			TargetGroup: &groups[3],
			Overhelm:    &varFalse,
			Factor:      0.99,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID21 = &discount.Discount{
		Name:        "Сокращатель 2",
		Description: "Скидка на сумму стоимостей товаров, принадлежащих сервису сокращателя, от 2500 р",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			PriceFrom: &priceFroms[5],
			Group:     &groups[3],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Group.Enum(),
			TargetGroup: &groups[3],
			Overhelm:    &varFalse,
			Factor:      0.96,
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}

	DiscountID22 = &discount.Discount{
		ID:          "641b2d73e0e07a7e90b59616",
		Name:        "Анлим Квиз 1",
		Description: "Скидка на количество дней безлимитного использования опросника, от 30 дней",
		Layer:       3,
		Condition: &discount.DiscountCondition{
			Term:    &terms[0],
			Product: &products[2],
		},
		Target: &discount.DiscountCalculationTarget{
			TargetScope: discount.TargetScope_Each.Enum(),
			TargetGroup: &groups[0],
			Overhelm:    &varFalse,
			Products: []*discount.ProductTarget{
				{
					ID:       products[2],
					Factor:   0.97,
					Overhelm: &varFalse,
				},
			},
		},
		Deprecated: false,
		Audit: &discount.Audit{
			CreatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			UpdatedAt: &timestamppb.Timestamp{Seconds: 1672444800},
			Deleted:   false,
		},
	}
)

var Discounts = []*discount.Discount{
	DiscountID1,
	DiscountID2,
	DiscountID3,
	DiscountID4,
	DiscountID5,
	DiscountID6,
	DiscountID7,
	DiscountID8,
	DiscountID9,
	DiscountID10,
	DiscountID11,
	DiscountID12,
	DiscountID13,
	DiscountID14,
	DiscountID15,
	DiscountID16,
	DiscountID17,
	DiscountID18,
	DiscountID19,
	DiscountID20,
	DiscountID21,
	DiscountID22,
}

var User1Discounts = []*discount.Discount{
	DiscountID1,
	DiscountID2,
	DiscountID3,
	DiscountID4,
	DiscountID5,
	DiscountID6,
	DiscountID8,
	DiscountID9,
	DiscountID10,
	DiscountID11,
	DiscountID12,
	DiscountID13,
	DiscountID14,
	DiscountID15,
	DiscountID16,
	DiscountID17,
	DiscountID18,
	DiscountID19,
	DiscountID20,
	DiscountID21,
	DiscountID22,
}
