package integration_test

import (
	"context"
	"fmt"
	"testing"

	"github.com/stretchr/testify/assert"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	"google.golang.org/protobuf/types/known/emptypb"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/proto/discount"
	"gitea.pena/PenaSide/discount/pkg/env"
	"gitea.pena/PenaSide/discount/tests/integration/data"
	"gitea.pena/PenaSide/discount/tests/integration/utils"
)

func TestGetAllDiscounts(t *testing.T) {
	config, err := env.Parse[core.Config]("../../.env.test")

	assert.NoError(t, err)
	assert.NotNil(t, config)

	discountClientConnection, err := grpc.Dial(
		fmt.Sprintf("%s:%s", config.GRPC.Host, config.GRPC.Port),
		grpc.WithTransportCredentials(insecure.NewCredentials()),
	)

	assert.NoError(t, err)

	discountClient := discount.NewDiscountServiceClient(discountClientConnection)

	t.Run("Получение всех скидок", func(t *testing.T) {
		response, err := discountClient.GetAllDiscounts(context.Background(), &emptypb.Empty{})

		isNoError := assert.NoError(t, err)
		isNotNil := assert.NotNil(t, response)

		if isNotNil && isNoError {
			utils.CleanDiscountsIDs(response.Discounts)
			assert.Equal(t, data.Discounts, response)
		}
	})
}
