package integration_test

import (
	"context"
	"fmt"
	"testing"

	"github.com/stretchr/testify/assert"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	"gitea.pena/PenaSide/discount/internal/core"
	"gitea.pena/PenaSide/discount/internal/proto/discount"
	"gitea.pena/PenaSide/discount/pkg/env"
	"gitea.pena/PenaSide/discount/tests/integration/data"
)

func TestGetDiscountByID(t *testing.T) {
	config, err := env.Parse[core.Config]("../../.env.test")

	assert.NoError(t, err)
	assert.NotNil(t, config)

	discountClientConnection, err := grpc.Dial(
		fmt.Sprintf("%s:%s", config.GRPC.Host, config.GRPC.Port),
		grpc.WithTransportCredentials(insecure.NewCredentials()),
	)

	assert.NoError(t, err)

	discountClient := discount.NewDiscountServiceClient(discountClientConnection)

	t.Run("Поиск существующей скидки", func(t *testing.T) {
		findedDiscount, err := discountClient.GetDiscountByID(context.Background(), &discount.GetDiscountByIDRequest{
			ID: "641b2d73e0e07a7e90b59616",
		})

		isNoError := assert.NoError(t, err)
		isNotNil := assert.NotNil(t, findedDiscount)

		if isNoError && isNotNil {
			assert.Equal(t,
				discount.Discount{
					ID:          data.DiscountID22.ID,
					Name:        data.DiscountID22.Name,
					Layer:       data.DiscountID22.Layer,
					Description: data.DiscountID22.Description,
					Condition:   data.DiscountID22.Condition,
					Deprecated:  data.DiscountID22.Deprecated,
				},
				discount.Discount{
					ID:          findedDiscount.ID,
					Name:        findedDiscount.Name,
					Layer:       findedDiscount.Layer,
					Description: findedDiscount.Description,
					Condition:   findedDiscount.Condition,
					Deprecated:  findedDiscount.Deprecated,
				},
			)
		}
	})

	t.Run("Поиск не существующей скидки", func(t *testing.T) {
		findedDiscount, err := discountClient.GetDiscountByID(context.Background(), &discount.GetDiscountByIDRequest{
			ID: "",
		})

		assert.Error(t, err)
		assert.Nil(t, findedDiscount)
	})
}
